package de.lmu.ifi.pst.x10080133.monitor;

import java.util.Properties;

import net.sf.json.JSONObject;
import net.sf.json.util.JSONUtils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Controller;

import de.lmu.ifi.pst.x10080133.domain.NavigationFile;
import de.lmu.ifi.pst.x10080133.domain.NavigationNode;
import de.lmu.ifi.pst.x10080133.domain.Rule;

/**
 * 
 * @author Roman.Schwienbacher
 */
@Controller
public class SecurityProvider implements ApplicationContextAware {

	private final Log log = LogFactory.getLog(SecurityProvider.class);

	public static final String SECURITY_FILENAME = "security.properties";
	public static final String NAVIGATION_PROPERTY = "navigation.file";

	private static NavigationFile navigationFile;

	/**
	 * Read and parse security.properties to NavigationFile instance
	 */
	@Override
	public void setApplicationContext(ApplicationContext arg0) throws BeansException { // called @ spring initialization

		log.debug("try to parse " + SECURITY_FILENAME);
				
		try {

			Properties properties = new Properties();
			properties.load(SecurityProvider.class.getResourceAsStream("/" + SECURITY_FILENAME));

			if (properties.containsKey("navigation.file")) {

				// read JSON content out of property
				if (JSONUtils.mayBeJSON(properties.getProperty(NAVIGATION_PROPERTY))) {

					JSONObject jsonHolder = JSONObject.fromObject(properties.getProperty(NAVIGATION_PROPERTY));
					navigationFile = (NavigationFile) JSONObject.toBean(jsonHolder, NavigationFile.class); // and parse to NavigationFile instance
					
					log.debug(SECURITY_FILENAME + " parsed successfully for application " + navigationFile.getApplication());
					log.debug("File generation date: " + navigationFile.get_comment());
					
				} else {
					log.error("Security file " + SECURITY_FILENAME + " contains no valid JSON");
				}
			} else {
				log.error("Security property " + NAVIGATION_PROPERTY + " not found");
			}
		} catch (Exception _e) {
			log.error("Error @parsing " + SECURITY_FILENAME, _e);
		}
	}
	
	/**
	 * 
	 * @return login location by home attribute
	 */
	public static String getHomeLocation() {
		
		String rtrn = "";
		
		for (NavigationNode node : navigationFile.getLocations()) {
			if (node.isHome()) {
				rtrn = node.getLocation();
			}
		}
		
		return rtrn;
	}
	
	/**
	 * Decides if user is allowed to access preVisitedLocationwith his userRoles and preVisitedLocation by navigationFile
	 * @param userRoles
	 * @param preVisitedLocation
	 * @param requestedLocation
	 * @return
	 */
	public static boolean isAllowedToAccess(String[] userRoles, String preVisitedLocation, String requestedLocation) {
		
		try {
			
			if (preVisitedLocation.equals(requestedLocation)) { // allow if preVisited equals requested location
				return true;
			}
			
			for (NavigationNode node : navigationFile.getLocations()) { // iterate over all defined nodes
				
				if (node != null && node.getLocation() != null && node.getLocation().equals(requestedLocation)) { // match node by requestedLocation
					
					for (Rule rule : node.getRules()) { // now iterate over all rules of the node
						
						for (String role : userRoles) { // iterate over all userRoles and compare with role of the rule
							
							if (rule.getRole().equals("*") || rule.getRole().equals(role)) { // check which userRole matches to rule 
								
								for (String preVisited : rule.getPre_visited()) { // finally check if preVisitedLocation of user is valid against the preVisited constraint of the rule
									
									if (preVisited != null && preVisited.equals(preVisitedLocation)) {
										return true;
									}
								}
							}
						}
						
					}					
				}				
			}			
			
		} catch (Exception _e) {
			_e.printStackTrace();			
		}
		
		return false;		
	}
	
	/**
	 * Gets violationLocation in case that access is not allowed corresponding to given params and navigationFile
	 * @param requestedLocation
	 * @return
	 */
	public static String getViolationLocation(String requestedLocation) {
		
		String rtrn = navigationFile.getDefault_violation(); // application specific violation as default
		
		try {
			
			for (NavigationNode node : navigationFile.getLocations()) { // iterate over all defined nodes
				
				if (node != null && node.getLocation() != null && node.getLocation().equals(requestedLocation)) { // match node by requestedLocation
					
					rtrn = node.getViolation(); // node specific violation
					break;
				}				
			}			
			
		} catch (Exception _e) {
			_e.printStackTrace();			
		}
		
		return rtrn;				
	}
}
