package com.ticketapplication.controller;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.faces.bean.ManagedBean;
import javax.faces.component.UIParameter;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;

import com.ticketapplication.dao.CustomerDAO;
import com.ticketapplication.dao.TicketDAO;
import com.ticketapplication.dao.UserDAO;
import com.ticketapplication.domain.Customer;
import com.ticketapplication.domain.Role;
import com.ticketapplication.domain.Ticket;
import com.ticketapplication.domain.User;

import de.lmu.ifi.pst.x10080133.monitor.AppMonitor;

/**
 * Controller which provides access to the DAO's and the security package for the view
 * 
 * @author Roman.Schwienbacher
 */
@Controller
@ManagedBean
public class TicketApplicationController implements Serializable {

	private final Log log = LogFactory.getLog(TicketApplicationController.class);

	private static final long serialVersionUID = -5320397963522290090L;

	// DAO's autowired by Spring
	
	@Autowired
	private TicketDAO ticketDAO;

	@Autowired
	private CustomerDAO customerDAO;

	@Autowired
	private UserDAO userDAO;

	
	// member variables
	
	private String clientMessage;
	private String clientSessionMessage;

	private boolean reloadAllTickets = true;
	private List<Ticket> allTickets = new ArrayList<Ticket>();
	private Ticket ticket = new Ticket();

	private List<Customer> allCustomers = new ArrayList<Customer>();

	private boolean reloadAllUsers = true;
	private List<User> allUsers = new ArrayList<User>();
	private User user = new User();

	private List<Role> allRoles = new ArrayList<Role>();

	
	/**
	 * reload all tickets from DB at next loading
	 * @param event
	 */
	public void reloadAllTicketsFromDbNextLoad(ActionEvent event) {
		reloadAllTickets = true;
	}

	/**
	 * 
	 * @return the list of all ticket instances
	 */
	public List<Ticket> readAllTickets() {
		return ticketDAO.readAllTickets();
	}

	/**
	 * instantiates ticket out of DB by editTicketId
	 * @param event
	 */
	public void editTicket(ActionEvent event) {

		clientMessage = "";

		try {

			int id = Integer.parseInt(((UIParameter) event.getComponent().findComponent("editTicketId")).getValue().toString());

			log.debug("try to edit ticket with id " + id);

			for (Ticket _ticket : allTickets) {
				if (_ticket.getId() == id) {
					ticket = _ticket;
					break;
				}
			}

			if (ticket != null) {
				log.debug("ticket initialized successfully");
			} else {
				log.debug("failed to initialize ticket by id " + id);
			}

		} catch (Exception _e) {
			_e.printStackTrace();
		}
	}

	/**
	 * instantiates plain new ticket
	 * @param event
	 */
	public void instantiateNewTicket(ActionEvent event) {

		ticket = new Ticket();
		ticket.setActive(true);
		clientMessage = "";
	}

	/**
	 * save actual ticket to DB
	 * @return redirect location
	 */
	public String saveTicket() {

		if (ticket != null && ticket.getCustomer() != null && ticket.getCustomer().getId() != 0) {

			// validate and save
			if (ticketDAO.saveOrUpdate(ticket)) {
				reloadAllTickets = true;
				return "/protected/confirmTicket?faces-redirect=true";
			} else {
				clientMessage = "save not successful";
				return "/protected/editCreateTicket";
			}

		} else {
			clientMessage = "a customer must be selected";
			return "/protected/editCreateTicket";
		}
	}

	/**
	 * delete ticket out of DB by deleteTicketId
	 * @param event
	 */
	public void deleteTicket(ActionEvent event) {

		ticket = null;

		try {

			int id = Integer.parseInt(((UIParameter) event.getComponent().findComponent("deleteTicketId")).getValue().toString());

			log.debug("try to delete ticket with id " + id);

			for (Ticket _ticket : allTickets) {
				if (_ticket.getId() == id) {
					ticket = _ticket;
					break;
				}
			}

			if (ticket != null) {
				log.debug("ticket initialized successfully");
				ticketDAO.deleteTicket(ticket);
				reloadAllTickets = true;
			} else {
				log.debug("failed to initialize ticket by id " + id);
			}

		} catch (Exception _e) {
			_e.printStackTrace();
		}
	}

	/**
	 * just redirect to userPostbox at ticket confirmation
	 * @return
	 */
	public String confirmTicket() {

		return "/protected/userPostbox?faces-redirect=true";

	}

	/**
	 * 
	 * @return all ticket instances out of DB
	 */
	@Transactional(readOnly = true)
	public List<Ticket> getAllTickets() {

		if (allTickets.isEmpty() || reloadAllTickets) {

			log.debug("read all tickets called by controller");

			synchronized (allTickets) {

				reloadAllTickets = false;
				allTickets.clear();
				allTickets.addAll(ticketDAO.readAllTickets());
			}
		}

		return allTickets;
	}

	/**
	 * 
	 * @return all customer instances out of DB
	 */
	@Transactional(readOnly = true)
	public List<Customer> getAllCustomers() {

		if (allCustomers.isEmpty()) {

			log.debug("read all customers called by controller");

			synchronized (allCustomers) {

				allCustomers.clear();
				allCustomers.addAll(customerDAO.readAllCustomers());
			}
		}

		return allCustomers;
	}

	/**
	 * assign customer to ticket
	 * @param event
	 */
	public void selectCustomer(ActionEvent event) {

		clientMessage = "";

		try {

			int ticketId = Integer.parseInt(((UIParameter) event.getComponent().findComponent("editTicketId")).getValue().toString());
			int customerId = Integer.parseInt(((UIParameter) event.getComponent().findComponent("customerId")).getValue().toString());

			log.debug("try add customer with id " + customerId + " to ticket with id " + ticketId);

			for (Ticket _ticket : getAllTickets()) {
				if (_ticket.getId() == ticketId) {
					ticket = _ticket;
					break;
				}
			}

			for (Customer _customer : getAllCustomers()) {
				if (_customer.getId() == customerId) {
					ticket.setCustomer(_customer);
					ticket.setCustomer_id(_customer.getId());
					break;
				}
			}

			if (ticket != null) {
				log.debug("ticket initialized successfully");
				reloadAllTickets = true;
			} else {
				log.debug("failed to initialize ticket by id " + ticketId);
			}

		} catch (Exception _e) {
			_e.printStackTrace();
		}
	}

	/**
	 * reload all user out of DB at next user loading
	 * @param event
	 */
	public void reloadAllUsersFromDbNextLoad(ActionEvent event) {
		reloadAllUsers = true;
	}

	/**
	 * read all user instances out of DB
	 * @return
	 */
	public List<User> readAllUsers() {
		return userDAO.readAllUsers();
	}

	/**
	 * instantiate user instance out of DB by editUserId
	 * @param event
	 */
	public void editUser(ActionEvent event) {

		clientMessage = "";

		try {

			int id = Integer.parseInt(((UIParameter) event.getComponent().findComponent("editUserId")).getValue().toString());

			log.debug("try to edit user with id " + id);

			for (User _user : allUsers) {
				if (_user.getId() == id) {
					user = _user;
					break;
				}
			}

			if (user != null) {
				log.debug("user initialized successfully");
			} else {
				log.debug("failed to initialize user by id " + id);
			}

		} catch (Exception _e) {
			_e.printStackTrace();
		}
	}

	/**
	 * instantiate plain new user
	 * @param event
	 */
	public void instantiateNewUser(ActionEvent event) {

		user = new User();
		user.setActive(true);
		clientMessage = "";
	}

	/**
	 * save current user instance to DB and redirect
	 * @return
	 */
	public String saveUser() {

		if (user != null && user.getRoles() != null && !user.getRoles().isEmpty()) {

			// validate and save
			if (userDAO.saveOrUpdate(user)) {
				reloadAllUsers = true;
				return "/protected/adminHome?faces-redirect=true";
			} else {
				clientMessage = "save not successful";
				return "/protected/editCreateUser";
			}

		} else {
			clientMessage = "a role must be selected";
			return "/protected/editCreateUser";
		}
	}

	/**
	 * delete user instance from DB by deleteUserId
	 * @param event
	 */
	public void deleteUser(ActionEvent event) {

		user = null;

		try {

			int id = Integer.parseInt(((UIParameter) event.getComponent().findComponent("deleteUserId")).getValue().toString());

			log.debug("try to delete user with id " + id);

			for (User _user : allUsers) {
				if (_user.getId() == id) {
					user = _user;
					break;
				}
			}

			if (user != null) {
				log.debug("user initialized successfully");
				userDAO.deleteUser(user);
				reloadAllUsers = true;
			} else {
				log.debug("failed to initialize user by id " + id);
			}

		} catch (Exception _e) {
			_e.printStackTrace();
		}
	}

	/**
	 * 
	 * @return all user instances out of DB
	 */
	@Transactional(readOnly = true)
	public List<User> getAllUsers() {

		if (allUsers.isEmpty() || reloadAllUsers) {

			log.debug("read all users called by controller");

			synchronized (allUsers) {

				reloadAllUsers = false;
				allUsers.clear();
				allUsers.addAll(userDAO.readAllUsers());
			}
		}

		return allUsers;
	}

	/**
	 * 
	 * @return all role instances out of  DB
	 */
	@Transactional(readOnly = true)
	public List<Role> getAllRoles() {

		if (allRoles.isEmpty()) {

			log.debug("read all roles called by controller");

			synchronized (allRoles) {

				allRoles.clear();
				allRoles.addAll(userDAO.readAllRoles());
			}
		}

		return allRoles;
	}

	public void setAllTickets(List<Ticket> allTickets) {
		this.allTickets = allTickets;
	}

	public Ticket getTicket() {
		return ticket;
	}

	public void setTicket(Ticket ticket) {
		this.ticket = ticket;
	}

	public void setAllUsers(List<User> allUsers) {
		this.allUsers = allUsers;
	}

	public User getUser() {
		return user;
	}

	public void setUser(User user) {
		this.user = user;
	}

	public TicketDAO getTicketDAO() {
		return ticketDAO;
	}

	public void setTicketDAO(TicketDAO ticketDAO) {
		this.ticketDAO = ticketDAO;
	}

	public CustomerDAO getCustomerDAO() {
		return customerDAO;
	}

	public void setCustomerDAO(CustomerDAO customerDAO) {
		this.customerDAO = customerDAO;
	}

	public UserDAO getUserDAO() {
		return userDAO;
	}

	public void setUserDAO(UserDAO userDAO) {
		this.userDAO = userDAO;
	}

	public String getClientMessage() {
		return clientMessage;
	}

	public void setClientMessage(String clientMessage) {
		this.clientMessage = clientMessage;
	}

	/**
	 * 
	 * @return session message by AppMonitor.MESSAGE_KEY
	 */
	public String getClientSessionMessage() {
		
		Map<String, Object> sessionMap = FacesContext.getCurrentInstance().getExternalContext().getSessionMap();
		if (sessionMap != null && sessionMap.containsKey(AppMonitor.MESSAGE_KEY)) {
			clientSessionMessage = sessionMap.get(AppMonitor.MESSAGE_KEY).toString();
		} 

		return clientSessionMessage;
	}

	public void setClientSessionMessage(String clientSessionMessage) {
		this.clientSessionMessage = clientSessionMessage;
	}
}
