package com.ticketapplication.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import com.ticketapplication.domain.Customer;

/**
 * DAO for Customer domain 
 * 
 * @author Roman.Schwienbacher 
 */
@Repository
public class CustomerDAO {

	private final Log log = LogFactory.getLog(CustomerDAO.class);

	@Autowired
	private JdbcTemplate jdbcTemplate;

	public Customer readCustomerById(int id) {

		log.debug("try to read customer by id " + id);

		Customer rtrn = null;

		try {

			rtrn = jdbcTemplate.queryForObject("SELECT * FROM customer WHERE id = ?", new RowMapper<Customer>() {

				@Override
				public Customer mapRow(ResultSet rs, int rownum) throws SQLException {

					Customer customer = new Customer();
					customer.setId(rs.getInt("id"));
					customer.setActive(rs.getBoolean("active"));
					customer.setFirstName(rs.getString("firstName"));
					customer.setLastName(rs.getString("lastName"));
					customer.setEmail(rs.getString("email"));
					customer.setBirthdate(rs.getDate("birthdate"));
					
					return customer;
				}
			}, new Object[] { id });

			log.debug("customer read successfully");

			return rtrn;
		} catch (Exception _e) {
			_e.printStackTrace();
		}

		return null;
	}

	public List<Customer> readAllCustomers() {

		log.debug("try to read all Customers");

		List<Integer> idsToRead = null;
		List<Customer> customers = new ArrayList<Customer>();

		try {

			idsToRead = jdbcTemplate.query("SELECT id FROM customer ORDER BY lastName, firstName", new RowMapper<Integer>() {

				@Override
				public Integer mapRow(ResultSet rs, int rownum) throws SQLException {
					return rs.getInt("id");
				}
			});

			for (Integer id : idsToRead) {
				customers.add(readCustomerById(id));
			}

			return customers;

		} catch (Exception _e) {
			_e.printStackTrace();
		}

		return null;
	}

	public JdbcTemplate getJdbcTemplate() {
		return jdbcTemplate;
	}

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
	}
}
