package com.ticketapplication.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.stereotype.Repository;

import com.ticketapplication.domain.Customer;
import com.ticketapplication.domain.Ticket;

/**
 * DAO for Ticket domain 
 * 
 * @author Roman.Schwienbacher 
 */
@Repository
public class TicketDAO {

	private final Log log = LogFactory.getLog(TicketDAO.class);

	@Autowired
	private JdbcTemplate jdbcTemplate;

	private SimpleJdbcInsert insertTicket;

	public boolean saveOrUpdate(Ticket ticket) {

		log.debug("try to save ticket " + ticket);

		boolean success = false;

		try {

			if (ticket.getId() == 0) { // insert new ticket into db

				// generate map by ticket bean
				Map<String, Object> params = new HashMap<String, Object>(9);
				params.put("name", ticket.getName());
				params.put("info", ticket.getInfo());
				params.put("customer_id", ticket.getCustomer().getId());
				params.put("active", ticket.isActive());

				ticket.setId(insertTicket.executeAndReturnKey(params).intValue());
				success = true;

				log.debug("ticket inserted with id " + ticket.getId());
			} else { // update ticket

				jdbcTemplate.update("UPDATE ticket SET name = ? , info = ?, customer_id = ? , active = ? " + " WHERE id = ?", new Object[] { ticket.getName(), ticket.getInfo(), ticket.getCustomer().getId(), ticket.isActive(), ticket.getId() });
				success = true;

				log.debug("ticket updated with id " + ticket.getId());
			}

		} catch (Exception _e) {
			_e.printStackTrace();
		}

		return success;
	}

	public boolean deleteTicket(Ticket ticket) {

		log.debug("try to delete ticket " + ticket);

		boolean success = false;

		try {
			// delete ticket
			jdbcTemplate.update("DELETE FROM ticket WHERE id = ?", new Object[] { ticket.getId() });
			success = true;

			log.debug("ticket deleted successfully");

		} catch (Exception _e) {
			_e.printStackTrace();
		}

		return success;
	}

	public Ticket readTicketById(int id) {

		log.debug("try to read ticket by id " + id);

		Ticket rtrn = null;
		Customer customer = null;

		try {

			rtrn = jdbcTemplate.queryForObject("SELECT id, name, info, active, customer_id FROM ticket WHERE id = ?", new RowMapper<Ticket>() {

				@Override
				public Ticket mapRow(ResultSet rs, int rownum) throws SQLException {

					Ticket ticket = new Ticket();
					ticket.setId(rs.getInt("id"));
					ticket.setActive(rs.getBoolean("active"));
					ticket.setInfo(rs.getString("info"));
					ticket.setName(rs.getString("name"));
					ticket.setCustomer_id(rs.getInt("customer_id"));					

					return ticket;
				}
			}, new Object[] { id });

			customer = jdbcTemplate.queryForObject("SELECT id, firstName, lastName, email, birthdate, active FROM customer WHERE id = ?", new RowMapper<Customer>() {

				@Override
				public Customer mapRow(ResultSet rs, int rownum) throws SQLException {

					Customer customer = new Customer();
					customer.setId(rs.getInt("id"));
					customer.setActive(rs.getBoolean("active"));
					customer.setFirstName(rs.getString("firstName"));
					customer.setLastName(rs.getString("lastName"));
					customer.setEmail(rs.getString("email"));
					customer.setBirthdate(rs.getDate("birthdate"));
					
					return customer;
				}
			}, new Object[] { rtrn.getCustomer_id() });
			
			rtrn.setCustomer(customer);

			log.debug("Ticket read successfully");

			return rtrn;
		} catch (Exception _e) {
			_e.printStackTrace();
		}

		return null;
	}

	public List<Ticket> readAllTickets() {

		log.debug("try to read all Tickets");
		
		List<Integer> idsToRead = null;
		List<Ticket> tickets = new ArrayList<Ticket>();		

		try {
			
			idsToRead = jdbcTemplate.query("SELECT id FROM ticket ORDER BY id", new RowMapper<Integer>() {

				@Override
				public Integer mapRow(ResultSet rs, int rownum) throws SQLException {
					return rs.getInt("id");
				}
			});

			for (Integer id : idsToRead) {
				tickets.add(readTicketById(id));
			}

			return tickets;

		} catch (Exception _e) {
			_e.printStackTrace();
		}

		return null;
	}

	public JdbcTemplate getJdbcTemplate() {
		return jdbcTemplate;
	}

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
		this.insertTicket = new SimpleJdbcInsert(jdbcTemplate).withTableName("ticket").usingGeneratedKeyColumns("id");
	}
}
