package com.ticketapplication.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.stereotype.Repository;

import com.ticketapplication.domain.Role;
import com.ticketapplication.domain.User;

/**
 * DAO for User domain 
 * 
 * @author Roman.Schwienbacher 
 */
@Repository
public class UserDAO {

	private final Log log = LogFactory.getLog(UserDAO.class);

	@Autowired
	private JdbcTemplate jdbcTemplate;

	private SimpleJdbcInsert insertUser;

	public boolean saveOrUpdate(User user) {

		log.debug("try to save user " + user);

		boolean success = false;

		try {

			if (user.getId() == 0) { // insert new user into db

				// generate map by user bean
				Map<String, Object> params = new HashMap<String, Object>(9);
				params.put("username", user.getUsername());
				params.put("password", user.getPassword());
				params.put("hiredate", user.getHiredate());
				params.put("active", user.isActive());

				user.setId(insertUser.executeAndReturnKey(params).intValue());

				for (Role role : user.getRoles()) {
					jdbcTemplate.update("INSERT INTO user_role (user_id, role_id) VALUES (?,?)", new Object[] { user.getId(), role.getId() });
				}

				success = true;

				log.debug("user inserted with id " + user.getId());
			} else { // update user

				jdbcTemplate.update("UPDATE user SET username = ? , password = ?, hiredate = ? , active = ? " + " WHERE id = ?", new Object[] { user.getUsername(), user.getPassword(), user.getHiredate(), user.isActive(), user.getId() });

				jdbcTemplate.update("DELETE FROM user_role where user_id = ?", new Object[] { user.getId() });

				for (Role role : user.getRoles()) {
					jdbcTemplate.update("INSERT INTO user_role (user_id, role_id) VALUES (?,?)", new Object[] { user.getId(), role.getId() });
				}

				success = true;

				log.debug("user updated with id " + user.getId());
			}

		} catch (Exception _e) {
			_e.printStackTrace();
		}

		return success;
	}

	public boolean deleteUser(User user) {

		log.debug("try to delete user " + user);

		boolean success = false;

		try {
			// delete user
			jdbcTemplate.update("DELETE FROM user WHERE id = ?", new Object[] { user.getId() });
			success = true;

			log.debug("user deleted successfully");

		} catch (Exception _e) {
			_e.printStackTrace();
		}

		return success;
	}

	public User readUserById(int id) {

		log.debug("try to read user by id " + id);

		User rtrn = null;
		List<Role> roles = null;

		try {

			rtrn = jdbcTemplate.queryForObject("SELECT id, username, password, hiredate, active FROM user WHERE id = ?", new RowMapper<User>() {

				@Override
				public User mapRow(ResultSet rs, int rownum) throws SQLException {

					User user = new User();
					user.setId(rs.getInt("id"));
					user.setActive(rs.getBoolean("active"));
					user.setUsername(rs.getString("username"));
					user.setPassword(rs.getString("password"));
					user.setHiredate(rs.getDate("hiredate"));

					return user;
				}
			}, new Object[] { id });

			roles = jdbcTemplate.query("SELECT id, name FROM role WHERE id IN (SELECT role_id FROM user_role WHERE user_id = ?)", new RowMapper<Role>() {

				@Override
				public Role mapRow(ResultSet rs, int rownum) throws SQLException {

					Role role = new Role();
					role.setId(rs.getInt("id"));
					role.setName(rs.getString("name"));

					return role;
				}
			}, new Object[] { rtrn.getId() });

			rtrn.getRoles().clear();
			rtrn.getRoles().addAll(roles);

			log.debug("User read successfully");

			return rtrn;
		} catch (Exception _e) {
			_e.printStackTrace();
		}

		return null;
	}

	public List<User> readAllUsers() {

		log.debug("try to read all Users");

		List<Integer> idsToRead = null;
		List<User> users = new ArrayList<User>();

		try {

			idsToRead = jdbcTemplate.query("SELECT id FROM user ORDER BY id", new RowMapper<Integer>() {

				@Override
				public Integer mapRow(ResultSet rs, int rownum) throws SQLException {
					return rs.getInt("id");
				}
			});

			for (Integer id : idsToRead) {
				users.add(readUserById(id));
			}

			return users;

		} catch (Exception _e) {
			_e.printStackTrace();
		}

		return null;
	}

	public List<Role> readAllRoles() {

		log.debug("try to read all Roles");

		List<Role> roles = new ArrayList<Role>();

		try {
			roles = jdbcTemplate.query("SELECT id, name FROM role WHERE id IN (SELECT role_id FROM user_role WHERE user_id = ?)", new RowMapper<Role>() {

				@Override
				public Role mapRow(ResultSet rs, int rownum) throws SQLException {

					Role role = new Role();
					role.setId(rs.getInt("id"));
					role.setName(rs.getString("name"));

					return role;
				}
			});
			
			return roles;
			
		} catch (Exception _e) {
			_e.printStackTrace();
		}
		
		return null;
	}

	public JdbcTemplate getJdbcTemplate() {
		return jdbcTemplate;
	}

	public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
		this.jdbcTemplate = jdbcTemplate;
		this.insertUser = new SimpleJdbcInsert(jdbcTemplate).withTableName("user").usingGeneratedKeyColumns("id");
	}
}
