package com.ticketapplication.security;

import java.util.Map;

import javax.faces.bean.ManagedBean;
import javax.faces.context.FacesContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;

import com.ticketapplication.dao.UserDAO;
import com.ticketapplication.domain.Role;
import com.ticketapplication.domain.User;

import de.lmu.ifi.pst.x10080133.monitor.AppMonitor;
import de.lmu.ifi.pst.x10080133.monitor.SecurityProvider;

/**
 * Manages the authentication handling of the WIS
 * 
 * @author Roman.Schwienbacher 
 */
@Controller
@ManagedBean
public class AuthenticationController {

	private final Log log = LogFactory.getLog(AuthenticationController.class);

	@Autowired
	private UserDAO userDAO;

	private User user = new User();

	private String clientMessage;

	
	/**
	 * handles the authentication and login process
	 * @return redirect location after login
	 */
	public String login() {

		log.debug("request for login: " + user.getUsername());

		String rtrn = "";
		boolean success = false;
		clientMessage = "";
		
		// searc user by username and compare password
		for (User persistentUser : userDAO.readAllUsers()) {
			if (persistentUser.getUsername().equals(user.getUsername())) {

				if (persistentUser.getPassword().equals(user.getPassword())) {
					user = persistentUser;
					success = true;
				}
				break;
			}
		}

		if (success) {
			
			Map<String, Object> sessionMap = FacesContext.getCurrentInstance().getExternalContext().getSessionMap();
			
			// convert user-roles into csv-string
			String rolesVal = "";
			for (Role role : user.getRoles()) {
				rolesVal += role.getName() + ";";
			}
			
			if (!rolesVal.isEmpty()) {
				rolesVal = rolesVal.substring(0, rolesVal.length() - 1);
			}
			
			// put roles-key and username-key into session
			sessionMap.put(AppMonitor.ROLES_KEY, rolesVal);
			sessionMap.put(AppMonitor.USERNAME_KEY, user.getUsername());
			
			// redirect to requested location before login
			if (sessionMap.containsKey(AppMonitor.REQ_LOCATION_KEY) && sessionMap.get(AppMonitor.REQ_LOCATION_KEY) != null && !sessionMap.get(AppMonitor.REQ_LOCATION_KEY).toString().isEmpty()) {
				rtrn = sessionMap.get(AppMonitor.REQ_LOCATION_KEY) + "?faces-redirect=true";
				sessionMap.remove(AppMonitor.REQ_LOCATION_KEY);
			} else {
				clientMessage = "login successful but no requested location";
				rtrn = SecurityProvider.getHomeLocation() + "?faces-redirect=true";
			}	
			
			log.debug("login successful");	
			
		} else {
			
			clientMessage = "login not successful";
			rtrn = SecurityProvider.getHomeLocation() + "?faces-redirect=true";
			
			log.debug("login not successful");
		}
		
		user = new User();
		
		return rtrn;
	}
	
	/**
	 * 
	 * @return username of logged in user
	 */
	public String getUsername() {
		return FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get(AppMonitor.USERNAME_KEY) == null ? "" :  FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get(AppMonitor.USERNAME_KEY).toString();
	}

	/**
	 * handles logout process by removing all information of the session
	 */
	public String logout() {
		
		log.debug("logout " + getUsername());
		
		FacesContext.getCurrentInstance().getExternalContext().getSessionMap().clear();
		return "/goodbye?faces-redirect=true";
	}	

	public User getUser() {
		return user;
	}

	public void setUser(User user) {
		this.user = user;
	}

	public UserDAO getUserDAO() {
		return userDAO;
	}

	public void setUserDAO(UserDAO userDAO) {
		this.userDAO = userDAO;
	}

	public String getClientMessage() {
		return clientMessage;
	}

	public void setClientMessage(String clientMessage) {
		this.clientMessage = clientMessage;
	}

}